/*******************************************************************************

  Intel Data Center Bridging (DCB) Software
  Copyright(c) 2007-2008 Intel Corporation.

  This program is free software; you can redistribute it and/or modify it
  under the terms and conditions of the GNU General Public License,
  version 2, as published by the Free Software Foundation.

  This program is distributed in the hope it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.

  The full GNU General Public License is included in this distribution in
  the file called "COPYING".

  Contact Information:
  e1000-eedc Mailing List <e1000-eedc@lists.sourceforge.net>
  Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497

*******************************************************************************/

#ifndef TLV_H
#define TLV_H

#define ADDR2TLVSTR(a) (a)[0], (a)[1], (a)[2], (a)[3], (a)[4], (a)[5]
#define TLVSTR "%02x%02x%02x%02x%02x%02x"   /* Localization OK */

#include "dcb_osdep.h"
#include "ports.h"

#define DCB_USER_PRIORITY_COUNT   8

/* Bit Mappers */
#define BIT0 0x01
#define BIT1 0x02
#define BIT2 0x04
#define BIT3 0x08
#define BIT4 0x10
#define BIT5 0x20
#define BIT6 0x40
#define BIT7 0x80

/* TLV types */
#define END_OF_LLDPDU_TLV 0
#define CHASSIS_ID_TLV    1
#define PORT_ID_TLV       2
#define TIME_TO_LIVE_TLV  3
#define ORG_SPECIFIC_TLV  127

#define TYPE_0   0
#define TYPE_1   1
#define TYPE_2   2
#define TYPE_3   3
#define TYPE_4   4
#define TYPE_5   5
#define TYPE_6   6
#define TYPE_7   7
#define TYPE_8   8
#define TYPE_127 127

/* Received TLV types */
#define RECEIVED_LLDP_TLV_TYPE0            0x0001
#define RECEIVED_LLDP_TLV_TYPE1            0x0002
#define RECEIVED_LLDP_TLV_TYPE2            0x0004
#define RECEIVED_LLDP_TLV_TYPE3            0x0008
#define RECEIVED_LLDP_TLV_TYPE4            0x0010
#define RECEIVED_LLDP_TLV_TYPE5            0x0020
#define RECEIVED_LLDP_TLV_TYPE6            0x0040
#define RECEIVED_LLDP_TLV_TYPE7            0x0080
#define RECEIVED_LLDP_TLV_TYPE8            0x0100
#define RECEIVED_LLDP_TLV_TYPE127_DCB_TLV  0x0200

#define RECEIVED_DCB_TLV_TYPE1 0x0001
#define RECEIVED_DCB_TLV_TYPE2 0x0002
#define RECEIVED_DCB_TLV_TYPE3 0x0003
#define RECEIVED_DCB_TLV_TYPE4 0x0004
#define RECEIVED_DCB_TLV_TYPE5 0x0005

/* Chassis subtypes */
#define CHASSIS_ID_CHASSIS_COMPONENT 1
#define CHASSIS_ID_INTERFACE_ALIAS   2
#define CHASSIS_ID_PORT_COMPONENT    3
#define CHASSIS_ID_MAC_ADDRESS       4
#define CHASSIS_ID_NETWORK_ADDRESS   5
#define CHASSIS_ID_INTERFACE_NAME    6
#define CHASSIS_ID_LOCALLY_ASSIGNED  7

/* Port subtype */
#define PORT_ID_INTERFACE_ALIAS  1
#define PORT_ID_PORT_COMPONENT   2
#define PORT_ID_MAC_ADDRESS      3
#define PORT_ID_NETWORK_ADDRESS  4
#define PORT_ID_INTERFACE_NAME   5
#define PORT_ID_AGENT_CIRCUIT_ID 6
#define PORT_ID_LOCALLY_ASSIGNED 7

/* DCB TLV Definitions */
#define DCB_CONTROL_TLV                1
#define DCB_PRIORITY_GROUPS_TLV        2
#define DCB_PRIORITY_FLOW_CONTROL_TLV  3
#define DCB_APPLICATION_TLV            5

/* DCB TLV App SubTypes */
#define DEFAULT_SUBTYPE   0
#define FCOE_SUBTYPE      0
#define FCOE_SUBTYPE_LEN  1

enum {
	CURRENT_PEER,
	LAST_PEER
};

struct dcb_control_info {
	u8  oper_version;
	u8  max_version;
	u32 seqno;
	u32 ackno;
};

/* DCB_TLV TYPE 1 byte offsets */
#define DCB_Type1_OPER_VERSION_OFFSET  0
#define DCB_Type1_MAX_VERSION_OFFSET   1
#define DCB_Type1_SEQNO_OFFSET         2
#define DCB_Type1_ACKNO_OFFSET         6

struct  dcb_tlv_header {
	u8  oper_version;
	u8  max_version;
	u8  ewe;
	u8  sub_type;
};

/* DCB_TLV HEADER byte offsets */
#define DCB_HEADER_OPER_VERSION_OFFSET  0
#define DCB_HEADER_MAX_VERSION_OFFSET   1
#define DCB_HEADER_EWE_OFFSET           2
#define DCB_HEADER_SUB_TYPE_OFFSET      3

struct dcb_pg_cfg {
	u8  bwg_percentages[8];  /* % of link BW per BWG */
	struct {
		/* BWG ID - 3, strict_prio 2 (1: LSP, 2: GSP, 3: reserved),
		 * reserved 3
		*/
		u8  byte1;
		u8  byte2;               /* percentage of BWG bandwidth */
	} pg_up_settings[8];     /* Index is user priority */
};

/* DCB_TLV TYPE 2 byte offsets */
#define DCB_Type2_PERCENTAGE_OFFSET  4
#define DCB_Type2_SETTINGS_OFFSET   12
#define BYTE1_OFFSET                 0
#define BYTE2_OFFSET                 1


struct dcb_pfc_cfg {
	u8 admin_map; /* bitmap of admin mode, bit position is user priority */
};

struct dcb_pg_info {
	struct dcb_tlv_header hdr;
	struct dcb_pg_cfg desired_result;
};

struct dcb_pfc_info {
	struct dcb_tlv_header hdr;
	struct dcb_pfc_cfg desired_result;
};

/* DCB_TLV TYPE 3 byte offset */
#define DCB_Type3_MAP_OFFSET  4

struct dcb_app_info {
	struct dcb_tlv_header hdr;
	u8 desired_result[];
};

/* DCB_TLV TYPE 5 byte offset */
#define DCB_Type5_APP_OFFSET  4

struct packed_tlv {
	u16 size;     /* Of the entire tlv block */
	u8 *tlv;      /* tlv block */
};

struct unpacked_tlv {
	u8  type;
	u16 length;
	u8  *info_string;
};

/* Organizationally Unique Identifier */
#define DCB_OUI_LEN     3
#define OUI_SUBTYPE_LEN 1

struct dcb_tlv {
	u8 oui[DCB_OUI_LEN];
	u8 oui_subtype;
};

struct packed_tlv *pack_tlv(struct unpacked_tlv *tlv);
struct unpacked_tlv *unpack_tlv(struct packed_tlv *tlv);

struct unpacked_tlv *destruct_unpacked_tlv(struct unpacked_tlv *tlv);
struct packed_tlv *destruct_packed_tlv(struct packed_tlv *tlv);
struct unpacked_tlv *create_tlv(void);

struct unpacked_tlv *construct_type0_tlv(void);
boolean_t is_validate_type0_tlv(struct unpacked_tlv *tlv);

struct unpacked_tlv *construct_type1_tlv(struct port *port);
boolean_t is_validate_type1_tlv(struct unpacked_tlv *tlv);

struct unpacked_tlv *construct_type2_tlv(struct port *port);
boolean_t is_validate_type2_tlv(struct unpacked_tlv *tlv);

struct unpacked_tlv *construct_type3_tlv(struct port *port);
boolean_t is_validate_type3_tlv(struct unpacked_tlv *tlv);

struct unpacked_tlv *construct_type127_tlv(struct port *port);
boolean_t is_validate_type127_tlv(struct unpacked_tlv *tlv);

struct unpacked_tlv *construct_control_tlv(struct port *port);
boolean_t is_validate_control_tlv(struct unpacked_tlv *tlv);

struct unpacked_tlv *construct_pg_tlv(struct port *port, boolean_t *success);
boolean_t is_validate_pg_tlv(struct unpacked_tlv *tlv);

struct unpacked_tlv *construct_pfc_tlv(struct port *port, boolean_t *success);
boolean_t is_validate_pfc_tlv(struct unpacked_tlv *tlv);

struct unpacked_tlv *construct_app_tlv(struct port *port, u32 sub_type,
					boolean_t *success);
boolean_t is_validate__tlv(struct unpacked_tlv *tlv);
int tlv_ok(struct unpacked_tlv *tlv);

#endif /* TLVS_H */
